''************************************************
''* VGACOLOUR_640x480_8x16_FG16-BG16.spin 8/2016 *
''* Author: Werner L. Schneider                  *
''* 256 Chars, 8x16 Pixel,                       *
''************************************************
''
''
'' Based on VGA_HiRes Text Driver from Chip Gracey
''
''
CON

' 640 x 480 @ 69Hz settings: 80 x 30 characters

    hp = 640      'horizontal pixels
    vp = 480      'vertical pixels
    hf = 24       'horizontal front porch pixels
    hs = 40       'horizontal sync pixels
    hb = 128      'horizontal back porch pixels
    vf = 9        'vertical front porch lines
    vs = 3        'vertical sync lines
    vb = 28       'vertical back porch lines
    hn = 1        'horizontal normal sync state (0|1)
    vn = 1        'vertical normal sync state (0|1)
    pr = 30       'pixel rate in MHz at 80MHz system clock (5MHz granularity)

' columns and rows

    cols = hp / 8
    rows = vp / 16     


VAR long cog[2]


PUB start(BasePin, ScreenPtr, CursorPtr, SyncPtr) : okay | i, j

'' Start VGA driver - starts two COGs
'' returns false if two COGs not available
''
''      BasePin = VGA starting pin (0, 8, 16, 24, etc.)
''
''      ScreenPtr = Pointer to 80x30 words containing Latin-1 codes and colors for
''              each of the 80x30 screen characters. The lower byte of the word
''              contains the Latin-1 code to display. The upper byte contains
''              the foreground colour in bits 11..8 and the background colour in
''              bits 15..12.
''
''              screen word example: %00011111_01000001 = "A", white on blue
''
''      CursorPtr = Pointer to 6 bytes which control the cursors:
''
''              bytes 0,1,2: X, Y, and MODE of cursor 0
''              bytes 3,4,5: X, Y, and MODE of cursor 1
''
''              X and Y are in terms of screen characters
''              (left-to-right, top-to-bottom)
''
''              MODE uses three bottom bits:
''
''                      %x00 = cursor off
''                      %x01 = cursor on
''                      %x10 = cursor on, blink slow
''                      %x11 = cursor on, blink fast
''                      %0xx = cursor is solid block
''                      %1xx = cursor is underscore
''
''              cursor example: 127, 63, %010 = blinking block in lower-right
''
''      SyncPtr = Pointer to long which gets written with -1 upon each screen
''              refresh. May be used to time writes/scrolls, so that chopiness
''              can be avoided. You must clear it each time if you want to see
''              it re-trigger.

    ' if driver is already running, stop it
    stop

    ' implant pin settings
    reg_vcfg := $200000FF + (BasePin & %111000) << 6
    i := $FF << (BasePin & %011000)
    j := BasePin & %100000 == 0
    reg_dira := i & j
    reg_dirb := i & !j

    ' implant CNT value to sync COGs to
    sync_cnt := cnt + $10000

    ' implant pointers
    longmove(@screen_base, @ScreenPtr, 2)
    font_base := @font

    ' implant unique settings and launch first COG
    vf_lines.byte := vf
    vb_lines.byte := vb
    font_part := 1
    cog[1] := cognew(@d0, SyncPtr) + 1

    ' allow time for first COG to launch
    waitcnt($2000 + cnt)

    ' differentiate settings and launch second COG
    vf_lines.byte := vf+4
    vb_lines.byte := vb-4
    font_part := 0
    cog[0] := cognew(@d0, SyncPtr) + 1

    ' if both COGs launched, return true
    if cog[0] and cog[1]
        return true
    ' else, stop any launched COG and return false
    stop


PUB stop | i

'' Stop VGA driver - frees two COGs

    repeat i from 0 to 1
        if cog[i]
            cogstop(cog[i]~ - 1)

PUB getfont : v

    v := @font
    

CON

    #1, scanbuff[80], colorbuff[80], scancode[2*80-1+3], maincode   'enumerate COG RAM usage

    main_size = $1F0 - maincode                             'size of main program

    hv_inactive = (hn << 1 + vn) * $0101                    'H,V inactive states


DAT

'*****************************************************
'* Assembly language VGA high-resolution text driver *
'*****************************************************

' This program runs concurrently in two different COGs.
'
' Each COG's program has different values implanted for front-porch lines and
' back-porch lines which surround the vertical sync pulse lines. This allows
' timed interleaving of their active display signals during the visible portion
' of the field scan. Also, they are differentiated so that one COG displays
' even four-line groups while the other COG displays odd four-line groups.
'
' These COGs are launched in the PUB 'start' and are programmed to synchronize
' their PLL-driven video circuits so that they can alternately prepare sets of
' four scan lines and then display them. The COG-to-COG switchover is seemless
' due to two things: exact synchronization of the two video circuits and the
' fact that all COGs' driven output states get OR'd together, allowing one COG
' to output lows during its preparatory state while the other COG effectively
' drives the pins to create the visible and sync portions of its scan lines.
' During non-visible scan lines, both COGs output together in unison.
'
' COG RAM usage: $000      = d0 - used to inc destination fields for indirection
'                $001-$050 = scanbuff - longs which hold 4 scan lines
'                $051-$0a0 = colorbuff - longs which hold colors for 80 characters
'                $0a1-$142 = scancode - stacked WAITVID/SHR for fast display
'                $143-$1EF = maincode - main program loop which drives display

                        org     0                               ' set origin to $000 for start of program

d0                      long    1 << 9                          ' d0 always resides here at $000, executes as NOP


' Initialization code and data - after execution, space gets reused as scanbuff

                        ' Move main program into maincode area

:move                   mov     $1EF, main_begin + main_size - 1
                        sub     :move,d0s0                      ' (do reverse move to avoid overwrite)
                        djnz    main_ctr,#:move

                        ' Build scanbuff display routine into scancode

:waitvid                mov     scancode+0, i0                  ' org   scancode
:shr                    mov     scancode+1, i1                  ' waitvid colorbuff+0, scanbuff+0
                        add     :waitvid, d1                    ' shr   scanbuff+0,#8
                        add     :shr, d1                        ' waitvid colorbuff+1, scanbuff+1
                        add     i0, d0s0                        ' shr   scanbuff+1,#8
                        add     i1, d0                          ' ...
                        djnz    scan_ctr, #:waitvid             ' waitvid colorbuff+cols-1, scanbuff+cols-1

                        mov     scancode+cols*2-1, i2           ' mov   vscl,#hf
                        mov     scancode+cols*2+0, i3           ' waitvid hvsync,#0
                        mov     scancode+cols*2+1, i4           ' jmp   #scanret

                        ' Init I/O registers and sync COGs' video circuits

                        mov     dira, reg_dira                  ' set pin directions
                        mov     dirb, reg_dirb
                        movi    frqa, #(pr / 5) << 2            ' set pixel rate
                        mov     vcfg, reg_vcfg                  ' set video configuration
                        mov     vscl, #1                        ' set video to reload on every pixel
                        waitcnt sync_cnt, colormask             ' wait for start value in cnt, add ~1ms
                        movi    ctra, #%00001_110               ' COGs in sync! enable PLLs now - NCOs locked!
                        waitcnt sync_cnt, #0                    ' wait ~1ms for PLLs to stabilize - PLLs locked!
                        mov     vscl, #100                      ' insure initial WAITVIDs lock cleanly

                        ' Jump to main loop

                        jmp     #vsync                          ' jump to vsync - WAITVIDs will now be locked!

                        ' Data

d0s0                    long    1 << 9 + 1
d1                      long    1 << 10
main_ctr                long    main_size
scan_ctr                long    cols

i0                      waitvid colorbuff+0, scanbuff+0
i1                      shr     scanbuff+0, #8
i2                      mov     vscl, #hf
i3                      waitvid hvsync, #0
i4                      jmp     #scanret

reg_dira                long    0                               ' set at runtime
reg_dirb                long    0                               ' set at runtime
reg_vcfg                long    0                               ' set at runtime
sync_cnt                long    0                               ' set at runtime

                        ' Directives

                        fit     scancode                        ' make sure initialization code and data fit
main_begin              org     maincode                        ' main code follows (gets moved into maincode)


' Main loop, display field - each COG alternately builds and displays four scan lines

vsync                   mov     x, #vs                          ' do vertical sync lines
                        call    #blank_vsync

vb_lines                mov     x, #vb                          ' do vertical back porch lines (# set at runtime)
                        call    #blank_vsync

                        mov     screen_ptr, screen_base         ' reset screen pointer to upper-left character

                        mov     row, #0                         ' reset row counter for cursor insertion
                        mov     fours, #rows * 4 / 2            ' set number of 4-line builds for whole screen

                        ' Build four scan lines into scanbuff

fourline                mov     font_ptr, font_part             ' get address of appropriate font section
                        shl     font_ptr, #8+2
                        add     font_ptr, font_base

                        movd    :pixa, #scanbuff-1              ' reset scanbuff address (pre-decremented)
                        movd    :cola, #colorbuff-1             ' reset colorbuff address (pre-decremented)
                        movd    :colb, #colorbuff-1

                        mov     y, #2                           ' must build scanbuff in two sections because
                        mov     vscl, vscl_line2x               ' ..pixel counter is limited to twelve bits

:halfrow                waitvid underscore, #0                  ' output lows to let other COG drive VGA pins
                        mov     x, #cols/2                      ' ..for 2 scan lines, ready for half a row

:column                 rdword  z, screen_ptr                   ' get character and colors from screen memory
                        mov     bg, z
                        and     z, #$ff                         ' mask character code
                        shl     z, #2                           ' * 4
                        add     z, font_ptr                     ' add font section address to point to 8*4 pixels

                        add     :pixa, d0                       ' increment scanbuff destination addresses
                        add     screen_ptr, #2                  ' increment screen memory address

:pixa                   rdlong  scanbuff, z                     ' read pixel long (8*4) into scanbuff

                        ror     bg, #12                         ' background color in bits 3..0
                        mov     fg, bg                          ' foreground color in bits 31..28
                        shr     fg, #28                         ' bits 3..0
                        add     fg, #fg_clut                    ' + offset to foreground CLUT
                        movs    :cola, fg
                        add     :cola, d0
                        add     bg, #bg_clut                    ' + offset to background CLUT
                        movs    :colb, bg
                        add     :colb, d0

:cola                   mov     colorbuff, 0-0
:colb                   or      colorbuff, 0-0

                        djnz    x, #:column                     ' another character in this half-row?

                        djnz    y, #:halfrow                    ' loop to do 2nd half-row, time for 2nd WAITVID

                        sub     screen_ptr, #2*cols             ' back up to start of same row in screen memory

                        ' Insert cursors into scanbuff

                        mov     z, #2                           ' ready for two cursors

:cursor                 rdbyte  x, cursor_base                  ' x in range?
                        add     cursor_base, #1
                        cmp     x, #cols        wc

                        rdbyte  y, cursor_base                  ' y match?
                        add     cursor_base, #1
                        cmp     y, row          wz

                        rdbyte  y, cursor_base                  ' get cursor mode
                        add     cursor_base, #1

        if_nc_or_nz     jmp     #:nocursor                      ' if cursor not in scanbuff, no cursor

                        add     x, #scanbuff                    ' cursor in scanbuff, set scanbuff address
                        movd    :xor, x

                        test    y, #%010        wc              ' get mode bits into flags
                        test    y, #%001        wz
        if_nc_and_z     jmp     #:nocursor                      ' if cursor disabled, no cursor

        if_c_and_z      test    slowbit, cnt    wc              ' if blink mode, get blink state
        if_c_and_nz     test    fastbit, cnt    wc

                        test    y, #%100        wz              ' get box or underscore cursor piece
        if_z            mov     x, longmask
        if_nz           mov     x, underscore
        if_nz           cmp     font_part, #3   wz              ' if underscore, must be last font section

:xor    if_nc_and_z     xor     scanbuff, x                     ' conditionally xor cursor into scanbuff

:nocursor               djnz    z, #:cursor                     ' second cursor?

                        sub     cursor_base, #3*2               ' restore cursor base

                        ' Display four scan lines from scanbuff

                        mov     y, #4                           ' ready for four scan lines

scanline                mov     vscl, vscl_chr                  ' set pixel rate for characters
                        jmp     #scancode                       ' jump to scanbuff display routine in scancode
scanret                 mov    vscl, #hs                        ' do horizontal sync pixels
                        waitvid hvsync, #1                      ' #1 makes hsync active
                        mov     vscl, #hb                       ' do horizontal back porch pixels
                        waitvid hvsync, #0                      ' #0 makes hsync inactive
                        shr     scanbuff+cols-1, #8             ' shift last column's pixels right by 8
                        djnz    y, #scanline                    ' another scan line?

                        ' Next group of four scan lines

                        add     font_part, #2                   ' if font_part + 2 => 4, subtract 4 (new row)
                        cmpsub  font_part, #4           wc      ' c=0 for same row, c=1 for new row
        if_c            add     screen_ptr, #2*cols             ' if new row, advance screen pointer
        if_c            add     row, #1                         ' if new row, increment row counter
                        djnz    fours, #fourline                ' another 4-line build/display?

                        ' Visible section done, do vertical sync front porch lines

                        wrlong  longmask,par                    ' write -1 to refresh indicator

vf_lines                mov     x,#vf                           ' do vertical front porch lines (# set at runtime)
                        call    #blank

                        jmp     #vsync                          ' new field, loop to vsync

                        ' Subroutine - do blank lines

blank_vsync             xor     hvsync,#$101                    ' flip vertical sync bits

blank                   mov     vscl, hx                        ' do blank pixels
                        waitvid hvsync, #0
                        mov     vscl, #hf                       ' do horizontal front porch pixels
                        waitvid hvsync, #0
                        mov     vscl, #hs                       ' do horizontal sync pixels
                        waitvid hvsync, #1
                        mov     vscl, #hb                       ' do horizontal back porch pixels
                        waitvid hvsync, #0
                        djnz    x,#blank                        ' another line?
blank_ret
blank_vsync_ret
                        ret

                        ' Data

screen_base             long    0                               ' set at runtime (3 contiguous longs)
cursor_base             long    0                               ' set at runtime
font_base               long    0                               ' set at runtime

font_part               long    0                               ' set at runtime

hx                      long    hp                              ' visible pixels per scan line
vscl_line               long    hp + hf + hs + hb               ' total number of pixels per scan line
vscl_line2x             long    (hp + hf + hs + hb) * 2         ' total number of pixels per 2 scan lines
vscl_chr                long    1 << 12 + 8                     ' 1 clock per pixel and 8 pixels per set
colormask               long    $FCFC                           ' mask to isolate R,G,B bits from H,V
longmask                long    $FFFFFFFF                       ' all bits set
slowbit                 long    1 << 25                         ' cnt mask for slow cursor blink
fastbit                 long    1 << 24                         ' cnt mask for fast cursor blink

underscore              long    $FFFF0000                       ' underscore cursor pattern

hv                      long    hv_inactive                     ' -H,-V states
hvsync                  long    hv_inactive ^ $200              ' +/-H,-V states


bg_clut                 long    %00000011_00000011              ' black           0
                        long    %00000011_01000011              ' dark red        1
                        long    %00000011_00010011              ' dark green      2
                        long    %00000011_01010011              ' dark yellow     3
                        long    %00000011_00000111              ' dark blue       4
                        long    %00000011_01000111              ' dark magenta    5
                        long    %00000011_00010111              ' dark cyan       6
                        long    %00000011_01010111              ' light gray      7
                        long    %00000011_10010011              ' light orange    8
                        long    %00000011_11000011              ' light red       9
                        long    %00000011_00110011              ' light green     10
                        long    %00000011_11110011              ' light yellow    11
                        long    %00000011_00001111              ' light blue      12
                        long    %00000011_11001111              ' light magenta   13
                        long    %00000011_00111111              ' light cyan      14
                        long    %00000011_11111111              ' white           15



fg_clut                 long    %00000011_00000011              ' black
                        long    %01000011_00000011              ' dark red
                        long    %00010011_00000011              ' dark green
                        long    %01010011_00000011              ' dark yellow
                        long    %00000111_00000011              ' dark blue
                        long    %01000111_00000011              ' dark magenta
                        long    %00010111_00000011              ' dark cyan
                        long    %01010111_00000011              ' light gray
                        long    %10010011_00000011              ' light orange
                        long    %11000011_00000011              ' light red
                        long    %00110011_00000011              ' light green
                        long    %11110011_00000011              ' light yellow
                        long    %00001111_00000011              ' light blue
                        long    %11001111_00000011              ' light magenta
                        long    %00111111_00000011              ' light cyan
                        long    %11111111_00000011              ' white



                        ' Uninitialized data

screen_ptr              res     1
font_ptr                res     1

x                       res     1
y                       res     1
z                       res     1
fg                      res     1
bg                      res     1

row                     res     1
fours                   res     1


'                        fit     $1f0


DAT

font  long

'First 4 Scanlines of Char 00h - 7Fh

long  $00000000,$00000000,$00000000,$00000000,$00000000,$00000000,$00000000,$00000000
long  $00000000,$00000000,$00000000,$00000000,$00000000,$80000000,$F75D2A14,$01000000
long  $00000000,$00000000,$00000000,$00000000,$80100080,$FCFDF8FA,$3FDF7FBF,$DDAC73CD
long  $FAFDFBFD,$3FBF1F2F,$02080400,$00000000,$00800080,$E0F88220,$FFFFFFF9,$FFFFFFFF
long  $00000000,$3C180000,$66666600,$36000000,$633E1818,$00000000,$361C0000,$0C0C0C00
long  $18300000,$180C0000,$00000000,$00000000,$00000000,$00000000,$00000000,$00000000
long  $361C0000,$1C180000,$633E0000,$633E0000,$38300000,$037F0000,$061C0000,$637F0000
long  $633E0000,$633E0000,$00000000,$FFFFFFFF,$FFFFFFFF,$FFFFFF77,$470F8922,$00000000
long  $3E000000,$1C080000,$663F0000,$663C0000,$361F0000,$667F0000,$667F0000,$663C0000
long  $63630000,$183C0000,$30780000,$66670000,$060F0000,$77630000,$67630000,$633E0000
long  $663F0000,$633E0000,$663F0000,$633E0000,$7E7E0000,$63630000,$63630000,$63630000
long  $63630000,$66660000,$637F0000,$00000000,$18181818,$00000000,$00000000,$00000000
long  $30180C00,$00000000,$06070000,$00000000,$30380000,$00000000,$6C380000,$00000000
long  $06070000,$18180000,$60600000,$06070000,$181C0000,$00000000,$00000000,$00000000
long  $00000000,$00000000,$00000000,$00000000,$0C080000,$00000000,$00000000,$00000000
long  $00000000,$00000000,$00000000,$00000000,$00000000,$18181818,$18181818,$18000000

'First 4 Scanlines of Char 80h - FFh

long  $00000000,$00000000,$00000000,$00000000,$00000000,$00000000,$FEFEFE00,$FFFFFF00
long  $FFFFFF00,$FFFFFF00,$FFFFFF00,$7F7F7F00,$00000000,$00000000,$00000000,$FFFFFFFF
long  $F0F0F0E0,$FFFFFFFF,$FFFFFFFF,$FFFFFFFF,$FFFFFFFF,$0F0F0F07,$FFFFFFFF,$FFFFFFFF
long  $F0C00000,$0F030000,$00000000,$00000000,$80808080,$00000000,$01010101,$FFFFFFFF
long  $80C0C0E0,$FFFFFFFF,$FFFFFFFF,$FFFFFFFF,$FFFFFFFF,$01030307,$0101030F,$00000000
long  $00000000,$00000000,$00000000,$00000000,$00000000,$00000000,$00000000,$8080C0F0
long  $00000000,$F0E00000,$FFFFFFE0,$FFFFFF07,$0F070000,$00000000,$00000000,$00000000
long  $000000C0,$07070E7F,$878F9EFC,$73F3E3E3,$001C393F,$00000000,$00000000,$00000000
long  $FEFEFEFE,$FFFFFFFF,$FFFFFFFF,$FFFFFFFF,$FFFFFFFF,$7F7F7F7F,$00000000,$00000000
long  $00000000,$00000000,$00000000,$00000000,$00000000,$00000000,$00000000,$00000000
long  $F8F8FCFC,$FFFFFFFF,$FFFFFFFF,$FFFFFFFF,$FFFFFFFF,$1F1F3F3F,$00000000,$00000000
long  $E0E0C0C0,$00000101,$9EBEBEEE,$E7EFCDDD,$397061E1,$00000000,$00000000,$00000000
long  $00000000,$00000000,$00000000,$00000000,$00000000,$00000000,$00000000,$00000000
long  $00000000,$00000000,$00000000,$00000000,$00000000,$00000000,$00000000,$00000000
long  $38387070,$3C7870E0,$0E0E1C1C,$38387077,$CECEDCDC,$F3F7BFBD,$00818183,$00010303
long  $00000000,$00000000,$00000000,$00000000,$00000000,$00000000,$00000000,$00000000

'Second 4 Scanlines of Char 00h - 7Fh

long  $00000000,$40000000,$812A0000,$0A000000,$00000000,$00000000,$09A00000,$2A005200
long  $AA00D500,$00020000,$00000000,$00000000,$C0000000,$FFFEFEF8,$D2BDD7EB,$7FFF3F1F
long  $02000000,$00000000,$00000000,$00000000,$10840028,$FEFEFDFC,$FF5F3FDF,$6CDBED32
long  $FBFDFAFF,$7F7F3FBF,$24011004,$00000000,$00000000,$00208401,$298014C0,$104200FF
long  $00000000,$18183C3C,$00000024,$36367F36,$603E0343,$18306343,$3B6E1C36,$00000006
long  $0C0C0C0C,$30303030,$FF3C6600,$7E181800,$00000000,$7F000000,$00000000,$18306040
long  $6B6B6363,$1818181E,$0C183060,$603C6060,$7F33363C,$603F0303,$633F0303,$18306060
long  $633E6363,$607E6363,$00001818,$480400FF,$128000FF,$A8401003,$00092200,$00000000
long  $7B7B6363,$7F636336,$663E6666,$03030343,$66666666,$161E1646,$161E1646,$7B030343
long  $637F6363,$18181818,$30303030,$1E1E3666,$06060606,$636B7F7F,$737B7F6F,$63636363
long  $063E6666,$63636363,$363E6666,$301C0663,$1818185A,$63636363,$63636363,$6B6B6363
long  $1C1C3E36,$183C6666,$0C183061,$FF000000,$18181818,$7F060400,$FE602000,$00000000
long  $00000000,$3E301E00,$66361E06,$03633E00,$33363C30,$7F633E00,$0C1E0C4C,$33336E00
long  $666E3606,$18181C00,$60607000,$1E366606,$18181818,$6B7F3700,$66663B00,$63633E00
long  $66663B00,$33336E00,$666E3B00,$06633E00,$0C0C3F0C,$33333300,$63636300,$6B636300
long  $1C366300,$63636300,$18337F00,$F8000000,$1F000000,$F8181818,$1F181818,$7E181818

'Second 4 Scanlines of Char 80h - FFh

long  $00000000,$F0E08000,$FFFFFFFE,$FFFFFF7F,$0F070100,$00000000,$FFFFFFFE,$FFFFFFFF
long  $FFFFFFFF,$FFFFFFFF,$FFFFFFFF,$FFFFFF7F,$00000000,$00000000,$00000000,$FFFFFFFF
long  $F0F0F0F0,$FFFFFFFF,$FFFFFFFF,$FFFFFFFF,$FFFFFFFF,$0F0F0F0F,$FFFFFFFF,$FFFFFFFF
long  $FCFCF8F8,$3F3F1F1F,$00000000,$00000000,$F0C08080,$00000000,$0F030101,$FFFFFFFF
long  $00000000,$FCFEFFFF,$FFFFFFFF,$FFFFFFFF,$3F7FFFFF,$00000000,$00000000,$00000000
long  $00000000,$00000000,$00000000,$00000000,$00000000,$00000000,$00000000,$00000000
long  $80000000,$FFFFFEF8,$FFFFFFFF,$FFFFFFFF,$FFFF7F1F,$01000000,$00000000,$00000000
long  $C0808000,$83838307,$FFE7C387,$38383971,$00000000,$00000000,$00000000,$00000000
long  $FFFFFFFE,$FFFFFFFF,$FFFFFFFF,$FFFFFFFF,$FFFFFFFF,$FFFFFF7F,$00000000,$00000000
long  $F0E00000,$FFFF0000,$07070000,$00000000,$00000000,$00000000,$00000000,$00000000
long  $C0E0E0F0,$FFFFFFFF,$FFFFFFFF,$FFFFFFFF,$FFFFFFFF,$0307070F,$00000000,$00000000
long  $002070E0,$00000000,$00868E8E,$0081C3E3,$00070F3F,$00000000,$00000000,$00000000
long  $80000000,$03000000,$00000000,$00000000,$00000000,$00000000,$00000000,$18000000
long  $00000000,$00000000,$00000000,$00000000,$00000000,$00000000,$00000000,$00000000
long  $00F0FCF8,$00030F1F,$0002070E,$00F0F8F8,$00CCCFCF,$003079F9,$00C0C0C0,$00000000
long  $00000000,$00000000,$00000000,$00000000,$00000000,$00000000,$00000000,$00000000

'Third 4 Scanlines of Char 00h - 7Fh

long  $00000000,$00501020,$29802208,$A20004A0,$00AA00AA,$087F2A40,$00154004,$00540180
long  $55005200,$00010204,$00000000,$00000000,$D0C4A890,$FF7F7FFF,$EFB57BAF,$FFFEFFFF
long  $43210408,$00000000,$00000000,$00000000,$40120882,$FFFFFFFF,$DF7FBFDF,$5437DBAD
long  $FDFBFAFD,$FFFFFF7F,$00220801,$00000000,$00000000,$00000000,$00000080,$0000004A
long  $00000000,$18180018,$00000000,$36367F36,$3E636160,$6163060C,$6E333333,$00000000
long  $30180C0C,$0C183030,$0000663C,$00001818,$18181800,$00000000,$18180000,$0103060C
long  $1C366363,$7E181818,$7F630306,$3E636060,$78303030,$3E636060,$3E636363,$0C0C0C0C
long  $3E636363,$1E306060,$00181800,$00000091,$00000048,$00000001,$00000000,$00000000
long  $3E033B7B,$63636363,$3F666666,$3C664303,$1F366666,$7F664606,$0F060606,$5C666363
long  $63636363,$3C181818,$1E333333,$67666636,$7F664606,$63636363,$63636363,$3E636363
long  $0F060606,$3E7B6B63,$67666666,$3E636360,$3C181818,$3E636363,$081C3663,$36777F6B
long  $6363363E,$3C181818,$7F634306,$00000000,$18181818,$0004067F,$002060FE,$00000000
long  $00000000,$6E333333,$3E666666,$3E630303,$6E333333,$3E630303,$1E0C0C0C,$3E333333
long  $67666666,$3C181818,$60606060,$6766361E,$3C181818,$636B6B6B,$66666666,$3E636363
long  $3E666666,$3E333333,$0F060606,$3E63301C,$386C0C0C,$6E333333,$1C366363,$367F6B6B
long  $63361C1C,$7E636363,$7F63060C,$18181818,$18181818,$00000000,$00000000,$0000183C

'Third 4 Scanlines of Char 80h - FFh

long  $00000000,$FFFFFEFC,$FFFFFFFF,$FFFFFFFF,$FFFF7F3F,$00000000,$FEFFFFFF,$FFFFFFFF
long  $FFFFFFFF,$FFFFFFFF,$FFFFFFFF,$7FFFFFFF,$FEFEFCF0,$FFFFFFFF,$7F7F3F0F,$00000000
long  $F0F0F0F0,$FFFFFFFF,$FFFFFFFF,$FFFFFFFF,$FFFFFFFF,$0F0F0F0F,$C0E0F8FE,$03071F7F
long  $F8F8FCFC,$1F1F3F3F,$00000000,$00000000,$FFFEFEFC,$00000000,$FF7F7F3F,$FFFFFFFF
long  $00000000,$0080E0F0,$FEFFFFFF,$7FFFFFFF,$0001070F,$00000000,$00000000,$00000000
long  $00000000,$F8F0C000,$01030300,$00000000,$00000000,$00000000,$00000000,$00000000
long  $F0E0E0C0,$FFFFFFFF,$FFFFFFFF,$FFFFFFFF,$FFFFFFFF,$0F070703,$00000000,$00000000
long  $40E0C0C0,$00000101,$00001E3F,$0000081C,$00000000,$00000000,$00000000,$00000000
long  $FEFFFFFF,$FFFFFFFF,$FFFFFFFF,$FFFFFFFF,$FFFFFFFF,$7FFFFFFF,$00000000,$00000000
long  $000000F0,$070707FF,$18000003,$63000000,$78000000,$00000000,$00000000,$00000000
long  $00000080,$F8FEFFFF,$FFFFFFFF,$FFFFFFFF,$1F7FFFFF,$00000001,$00000000,$00000000
long  $00000000,$00000000,$00000000,$00000000,$00000000,$00000000,$00000000,$00000000
long  $C0C0C080,$717D3F0F,$00000000,$00000000,$00000000,$00000000,$00000000,$0E0E1C1C
long  $00000000,$00000000,$00000000,$00000000,$00000000,$00000000,$00000000,$00000000
long  $00000000,$00000000,$00000000,$00000000,$00000000,$00000000,$00000000,$00000000
long  $00000000,$00000000,$00000000,$00000000,$00000000,$00000000,$00000000,$00000000

'Fourth 4 Scanlines of Char 00h - 7Fh

long  $00000000,$00008020,$0000AA00,$00000208,$00000000,$08140800,$00000000,$00000000
long  $00000000,$00000000,$00000000,$20800080,$FAF8F4F2,$7F9F7FDF,$5B9D77AA,$FFFEFFFF
long  $4F07430B,$00010000,$00000000,$00800000,$42084209,$0020421F,$28042200,$00A90022
long  $00441102,$11048278,$0492004A,$00000000,$00000000,$00000000,$00000000,$00000000
long  $00000000,$00000000,$00000000,$00000000,$00001818,$00000000,$00000000,$00000000
long  $00000000,$00000000,$00000000,$00000000,$0000000C,$00000000,$00000000,$00000000
long  $00000000,$00000000,$00000000,$00000000,$00000000,$00000000,$00000000,$00000000
long  $00000000,$00000000,$00000000,$00000000,$00000000,$00000000,$00000000,$00000000
long  $00000000,$00000000,$00000000,$00000000,$00000000,$00000000,$00000000,$00000000
long  $00000000,$00000000,$00000000,$00000000,$00000000,$00000000,$00000000,$00000000
long  $00000000,$00007030,$00000000,$00000000,$00000000,$00000000,$00000000,$00000000
long  $00000000,$00000000,$00000000,$00000000,$18181818,$00000000,$00000000,$00FF0000
long  $00000000,$00000000,$00000000,$00000000,$00000000,$00000000,$00000000,$001E3330
long  $00000000,$00000000,$003C6666,$00000000,$00000000,$00000000,$00000000,$00000000
long  $000F0606,$00783030,$00000000,$00000000,$00000000,$00000000,$00000000,$00000000
long  $00000000,$001F3060,$00000000,$18181818,$18181818,$00000000,$00000000,$00000000

'Fourth 4 Scanlines of Char 80h - FFh

long  $E0C0C080,$FFFFFFFF,$FFFFFFFF,$FFFFFFFF,$FFFFFFFF,$07030301,$00FEFEFE,$00FFFFFF
long  $00FFFFFF,$00FFFFFF,$00FFFFFF,$007F7F7F,$FFFFFFFF,$FFFFFFFF,$FFFFFFFF,$00000000
long  $E0F0F0F0,$FFFFFFFF,$FFFFFFFF,$FFFFFFFF,$FFFFFFFF,$070F0F0F,$00008080,$00000101
long  $0000C0F0,$0000030F,$00000000,$00000000,$FFFFFFFF,$00000000,$FFFFFFFF,$FFFFFFFF
long  $00000000,$00000000,$00000000,$00000000,$00000000,$00000000,$00000000,$00000000
long  $80800000,$FFFF1C3C,$F8E00000,$E3C10000,$3F3C0000,$00000000,$00000000,$00000000
long  $FCFCF8F8,$FFFFFFFF,$FFFFFFFF,$FFFFFFFF,$FFFFFFFF,$3F3F1F1F,$00000000,$00000000
long  $00000000,$00000000,$00000000,$00000000,$00000000,$00000000,$00000000,$00000000
long  $FEFEFEFE,$FFFFFFFF,$FFFFFFFF,$FFFFFFFF,$FFFFFFFF,$7F7F7F7F,$00000000,$00000000
long  $C0808080,$01030303,$FCDC9C9C,$DBBB7373,$E3E7FFFE,$00000000,$00000000,$00000000
long  $00000000,$0000E0F0,$E0FFFFFF,$07FFFFFF,$0000070F,$00000000,$0F030101,$00000000
long  $00000000,$00000000,$00000000,$00000000,$00000000,$00000000,$00000000,$F0C08080
long  $70E0E0E0,$E0E0E0E0,$7CF8F8B0,$E7CF8F0F,$DCBFBF1E,$797B7363,$07070E0E,$0707070E
long  $00000000,$00000000,$00000000,$00000000,$00000000,$00000000,$00000000,$00000000
long  $00000000,$00000000,$00000000,$00000000,$00000000,$00000000,$00000000,$00000000
long  $00000000,$00000000,$00000000,$00000000,$00000000,$00000000,$00000000,$00000000




{{
+------------------------------------------------------------------------------------------------------------------------------+
|                                   TERMS OF USE: Parallax Object Exchange License                                             |
+------------------------------------------------------------------------------------------------------------------------------+
|Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation    |
|files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,    |
|modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software|
|is furnished to do so, subject to the following conditions:                                                                   |
|                                                                                                                              |
|The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.|
|                                                                                                                              |
|THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE          |
|WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR         |
|COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,   |
|ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         |
+------------------------------------------------------------------------------------------------------------------------------+
}}